@@ -1,5 +1,5 @@
 (* Unison file synchronizer: src/main.ml *)
-(* Copyright 1999-2017, Benjamin C. Pierce
+(* Copyright 1999-2018, Benjamin C. Pierce
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -90,7 +90,7 @@ let socket =
     (fun _ -> fun i ->
       (try
          Some(int_of_string i)
-       with Failure "int_of_string" ->
+       with Failure _ ->
          raise(Prefs.IllegalValue "-socket must be followed by a number")))
     (function None -> [] | Some(i) -> [string_of_int i]) ;;
 
