@@ -1,5 +1,5 @@
 (* Unison file synchronizer: src/ubase/util.mli *)
-(* Copyright 1999-2017, Benjamin C. Pierce (see COPYING for details) *)
+(* Copyright 1999-2018, Benjamin C. Pierce (see COPYING for details) *)
 
 (* Miscellaneous utility functions and datatypes *)
 
@@ -55,7 +55,7 @@ val replacesubstrings : string -> (string * string) list -> string
 val concatmap : string -> ('a -> string) -> 'a list -> string
 val removeTrailingCR : string -> string
 val trimWhitespace : string -> string
-val splitIntoWords : string -> char -> string list
+val splitIntoWords : ?esc:char -> string -> char -> string list
 val splitIntoWordsByString : string -> string -> string list
 val padto : int -> string -> string
 
