@@ -1,5 +1,5 @@
 (* Unison file synchronizer: src/uitext.ml *)
-(* Copyright 1999-2017, Benjamin C. Pierce
+(* Copyright 1999-2018, Benjamin C. Pierce
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -174,11 +174,26 @@ let rec selectAction batch actions tryagain =
                      ^ "': try again  [type '?' for help]\n");
           tryagainOrLoop()
   in
-  doAction (match batch with
+  let handleExn s =
+    (* Make sure that the error messages start on their own lines and not
+     * after the prompt. *)
+    alwaysDisplay "\n";
+    raise (Util.Fatal ("Failure reading from the standard input ("^s^")\n"))
+  in
+  try doAction (match batch with
     None   ->
       summarizeChoices();
       getInput ()
   | Some i -> i)
+  with
+    (* Restart an interrupted system call (which can happen notably when
+     * the process is put in the background by SIGTSTP). *)
+    Unix.Unix_error (Unix.EINTR, _, _) -> selectAction batch actions tryagain
+    (* Simply print a slightly more informative message than the exception
+     * itself (e.g. "Uncaught unix error: read failed: Resource temporarily
+     * unavailable" or "Uncaught exception End_of_file"). *)
+  | End_of_file -> handleExn "End of file"
+  | Unix.Unix_error (err, _, _) -> handleExn (Unix.error_message err)
 
 let alwaysDisplayErrors prefix l =
   List.iter
@@ -230,6 +245,13 @@ let interact rilist =
   let (host1, host2) = root2hostname r1, root2hostname r2 in
   if not (Prefs.read Globals.batch) then display ("\n" ^ Uicommon.roots2string() ^ "\n");
   let rec loop prev =
+    let rec previous prev ril =
+      match prev with
+        ({ replicas = Problem s } as pri)::pril ->
+          displayri pri; display "\n"; display s; display "\n";
+          previous pril (pri::ril)
+      | pri::pril -> loop pril (pri::ril)
+      | [] -> loop prev ril in
     function
       [] -> (ConfirmBeforeProceeding, Safelist.rev prev)
     | ri::rest as ril ->
@@ -347,9 +369,7 @@ let interact rilist =
                   ("go back to previous item"),
                   (fun () ->
                      newLine();
-                     match prev with
-                       [] -> repeat()
-                     | prevri::prevprev -> loop prevprev (prevri :: ril)));
+                     previous prev ril));
                  (["g"],
                   ("proceed immediately to propagating changes"),
                   (fun() ->
@@ -798,8 +818,8 @@ let rec synchronizeUntilDone () =
   let repeatinterval =
     if Prefs.read Uicommon.repeat = "" then -1 else
     try int_of_string (Prefs.read Uicommon.repeat)
-    with Failure "int_of_string" ->
-      (* If the 'repeat' pref is not a number, switch modes... *)
+    with Failure _ ->
+      (* If the 'repeat' pref is not a valid number, switch modes... *)
       if Prefs.read Uicommon.repeat = "watch" then
         synchronizePathsFromFilesystemWatcher()
       else
@@ -824,11 +844,7 @@ let handleException e =
   restoreTerminal();
   let msg = Uicommon.exn2string e in
   Trace.log (msg ^ "\n");
-  if not !Trace.sendLogMsgsToStderr then begin
-    alwaysDisplay "\n";
-    alwaysDisplay msg;
-    alwaysDisplay "\n";
-  end
+  if not !Trace.sendLogMsgsToStderr then alwaysDisplay ("\n" ^ msg ^ "\n")
 
 let rec start interface =
   if interface <> Uicommon.Text then
